#include <p33fj256gp710.h>
#include "PCM_X2.h"

// I/O definitions
#define SYNC    _LATG0  // output 
#define SDO     _RF8    // SPI1 SDO


#define _FAR __attribute__(( far))
#define Nop() {__asm__ volatile ("nop");}

volatile int count;

//-------------------------------------------INTERRUPT ROUTINES----------------------------------------
void _ISRFAST _T2Interrupt( void)
{
	//can't do any work here, interrupt too short
	Nop();
    _T2IF = 0; // clear the interrupt flag

} // T2Interrupt

void _ISRFAST _T3Interrupt( void)
{
	//can't do any work here, interrupt too short
	Nop();
    _T3IF = 0; // clear the interrupt flag

} // T3Interrupt

void _ISRFAST _OC1Interrupt( void)
{
   //can't do any work here, interrupt too short
	Nop();
    _OC1IF = 0; // clear the interrupt flag

} // OC1Interrupt

void _ISRFAST _OC2Interrupt( void)
{
   //can't do any work here, interrupt too short
	Nop();
    _OC2IF = 0; // clear the interrupt flag

} // OC2Interrupt

void _ISRFAST _IC1Interrupt( void)
{
	count = 66;
	if(_RD8 == 1)
	{
		while (count > 0)
		count--;
	
		Nop();
		TMR3 = 4;					//clock run-in has begun
		TMR2 = 0;
		
		T3CON = 0x8000;				// ACTIVATE the clock, prescaler 1:1, internal clock
		T2CON = 0x8000;
		OC1CON = 0x000D;			// ACTIVATE OC1, initialize low
		OC2CON = 0x0005;
	}	

	else
	{
		OC1CON = 0x0000;			// disable OC1
		OC2CON = 0x0000;
		
		T3CON = 0x0000;				// disable clock, prescaler 1:1, internal clock
		T2CON = 0x0000;
	}

    _IC1IF = 0; // clear the interrupt flag

} // IC1Interrupt

void _ISRFAST _IC2Interrupt( void)
{
	//nothing done here
    _IC2IF = 0; // clear the interrupt flag

} // IC2Interrupt

//-------------------------------------------END OF INTERRUPT ROUTINES----------------------------------------
//------------------------------------------------------------------------------------------------------------
//---------------------------------------START OF INITIALIZATION ROUTINES-------------------------------------
void initSys( void)
{
	//Set Oscillator Configuration for 40MIPS
	OSCCON = 0x0300;			//
	CLKDIV = 0x0000;			//
	PLLFBD = 0x0026;			//
	
    // set the priority levels
	_T3IP  = 4; 				// this is the default value anyway
	_OC1IP = 4;		
    _OC2IP = 4;		
	_OC3IP = 4;
    _OC4IP = 4;

    //init TMR3 to provide timebase for data clock
	T3CON = 0x0000;				// disabled, prescaler 1:1, internal clock 	
	PR3 = CLOCK_PERIOD-1;

	// set initial duty cycles for OC1 module
	OC1R  = (CLOCK_PERIOD/2);
	OC1RS = (CLOCK_PERIOD/2);

	//init TMR2 to provide timebase for frame sync
	T2CON = 0x0000;				// disabled, prescaler 1:1, internal clock 	
	PR2 = 255;

	// set initial duty cycles for OC2 module
	OC2R  = 239;
	OC2RS = 255;

	//setup IC Modules
	IC1CON = 0x0001;			//capture every edge
	IC2CON = 0x0003;			//capture every rising edge

	// init Interrupts, clear the flag
	_OC1IF = 0;	_OC1IE = 0;		
	_OC2IF = 0;	_OC2IE = 0;
    _IC1IF = 0; _IC1IE = 1;	
	_IC2IF = 0; _IC2IE = 0;	
	_T3IF  = 0; _T3IE  = 0;	
	_T2IF  = 0; _T2IE  = 0;	
    	
	// init the processor priority level
	_IPL = 0;					// this is the default value anyway
    
    // init PORTS
    _TRISG0 = 0;            	// output the clock pin
	_TRISD8 = 1;				//make IC1 = RD8 pin an input
	_TRISD9 = 1;				//make IC2 = RD9 pin an input

} // initSys

//---------------------------------------END OF INITIALIZATION ROUTINES---------------------------------------
//------------------------------------------------------------------------------------------------------------
//-------------------------------------------------- MAIN ----------------------------------------------------
int main()
{

    // initializations
    initSys();    	// initialize system

	while(1)
	{
		//main loop
	}

	return 0;
	
} // main
//------------------------------------------------------------------------------------------------------------
